@@ -1,7 +1,7 @@
 #!@SH@
 ##
 ##  Shiela - CVS Access Control and Logging Facility
-##  Copyright (c) 2000 Ralf S. Engelschall <rse@engelschall.com>
+##  Copyright (c) 2000-2002 Ralf S. Engelschall <rse@engelschall.com>
 ##
 ##  This file is part of Shiela, an access control and logging
 ##  facility for Concurrent Versions System (CVS) repositories
@@ -25,7 +25,7 @@
 ##  shiela-install: Shiela repository install program (syntax: Bourne-Shell)
 ##
 
-version="0.9.1"
+version="0.9.2 (19-Aug-2002)"
 
 prefix="@prefix@"
 bindir="@bindir@"
@@ -272,25 +272,24 @@ if [ ".$V_tool_perl" = . ]; then
     V_tool_perl="perl"
     rm -f $tmpfile
     touch $tmpfile
-    c=0
     found=0
+    pc=99
     for dir in $paths; do
         dir=`echo $dir | sed -e 's;/*$;;'`
-        for perl in perl5 perl miniperl; do
-             if [ $minusx "$dir/$perl" ] && [ ! -d "$dir/$perl" ]; then
-                 perl="$dir/$perl"
-                 version=`$perl -v | grep version |\
-                          sed -e 's/.* version //' -e 's/ built.*//' -e 's/ with.*//'`
-                 versionnum="`echo $version | sed -e 's/\.//g' -e 's/_//g'`"
-                 versionnum=`expr $versionnum - $c`
-                 echo "$versionnum $perl" >>$tmpfile
+        nc=99
+        for name in perl perl5 miniperl; do
+             if [ $minusx "$dir/$name" -a ! -d "$dir/$name" ]; then
+                 perl="$dir/$name"
+                 pv=`$perl -e 'printf("%.3f", $]);'`
+                 echo "$pv:$pc:$nc:$perl" >>$tmpfile
                  found=1
              fi
+             nc=`expr $nc - 1`
         done
-        c=`expr $c + 1`
+        pc=`expr $pc - 1`
     done
     if [ $found = 1 ]; then
-        V_tool_perl="`cat $tmpfile | sort -u | tail -1 | cut '-d ' -f2`"
+        V_tool_perl="`cat $tmpfile | sort -u | tail -1 | cut -d: -f4`"
     fi
     rm -f $tmpfile
 fi
