@@ -1,7 +1,7 @@
 #!@PERL@ -w
 ##
 ##  Shiela - CVS Access Control and Logging Facility
-##  Copyright (c) 2000 Ralf S. Engelschall <rse@engelschall.com>
+##  Copyright (c) 2000-2002 Ralf S. Engelschall <rse@engelschall.com>
 ##
 ##  This file is part of Shiela, an access control and logging
 ##  facility for Concurrent Versions System (CVS) repositories
@@ -25,7 +25,7 @@
 ##  shiela: Shiela control program (syntax: Perl)
 ##
 
-my $version = '0.9.1';
+my $version = '0.9.2 (19-Aug-2002)';
 
 require 5.005;
 
@@ -125,7 +125,7 @@ sub rt_determine_one {
 
     #   supplied arguments
     $RT->{cvsroot} = $ENV{CVSROOT} || die 'unknown CVS root (set $CVSROOT variable)';
-    $RT->{userid}  = $ENV{LOGNAME} || $ENV{LOGUSER} || $ENV{USER} || die 'unknown CVS user';
+    $RT->{userid}  = $ENV{CVSUSER} || $ENV{LOGNAME} || $ENV{LOGUSER} || $ENV{USER} || die 'unknown CVS user';
 
     #   various directory paths
     $RT->{tmpdir}   = $ENV{TMPDIR} || $ENV{TEMPDIR} || '/tmp';
@@ -146,7 +146,7 @@ sub rt_determine_two {
 
     #   determine user information
     $RT->{username} = $CF->{Project}->{User}->{$RT->{userid}}->{name} ||
-                      die "unknown user `$RT->{userid}'";
+                      die "CVS user `$RT->{userid}' not found in Shiela configuration";
     $RT->{usermail} = $CF->{Project}->{User}->{$RT->{userid}}->{mail} ||
                       "$RT->{userid}\@localhost";
 
@@ -190,10 +190,10 @@ sub rt_determine_two {
     #   determine CVS version and capabilities
     my $v = `$RT->{cvs} --version 2>/dev/null`;
     $RT->{cvsvers} = '?';
-    $RT->{cvsvers} = $1 if ($v =~ m|Concurrent\s+Versions\s+System\s+\(CVS\)\s+([\d.]+)\s+|s);
+    $RT->{cvsvers} = $1 if ($v =~ m|Concurrent\s+Versions\s+System\s+\(CVS\)\s+([\d.p]+)\s+|s);
     $RT->{cvsossp} = 0;
     $RT->{cvsossp} = 1 if ($v =~ m|OSSP|s);
-    die "$RT->{cvs} is not at least CVS 1.10" if ($RT->{cvsvers} !~ m|^1\.10|);
+    die "$RT->{cvs} is not at least CVS 1.10" if ($RT->{cvsvers} !~ m|^1\.1[0-9]|);
     $RT->{useserver} = 0;
     $RT->{useserver} = 1 if ($v =~ m|server|s);
 
@@ -980,7 +980,12 @@ sub hook_taginfo {
 
     #   take the arguments
     my ($tagname, $tagop, $cvsdir, %cvsfiles) = @{$PA->{ARG}};
+
+    #   strip absolute prefix
     $cvsdir =~ s|^$RT->{cvsroot}/?||;
+    my $cvsdirphysical = `cd $RT->{cvsroot} && pwd`;
+    $cvsdirphysical =~ s|\n$||s; 
+    $cvsdir =~ s|^$cvsdirphysical/?||;
 
     #   provide access control 
     my @paths = ();
@@ -1182,7 +1187,8 @@ sub hook_commitinfo {
     my @newfiles = ();
     foreach my $cvsfile (@cvsfiles) {
         my $branch = 'HEAD';
-        if ($cvsstat =~ m|===+\nFile:\s+$cvsfile.+?Sticky Tag:\s+(\S+)|s) {
+        my $cvsfile_quoted = quotemeta($cvsfile);
+        if ($cvsstat =~ m|===+\nFile:\s+${cvsfile_quoted}.+?Sticky Tag:\s+(\S+)|s) {
             $branch = $1;
             $branch = 'HEAD' if ($branch eq '(none)');
         }
@@ -1402,7 +1408,7 @@ sub hook_loginfo_collect {
                     print STDERR "cvs import: Ignoring this operation - don't expect log messages!\n";
                     exit(0);
                 }
-                open(CVSS, "$RT->{cvs} -f -l -Q -n log -r$It $Is|");
+                open(CVSS, "$RT->{cvs} -f -l -Q -n log -r$It '$Is'|");
                 $rcslog = $_ while (<CVSS>);
                 close(CVSS);
             }
@@ -1487,8 +1493,8 @@ sub hook_loginfo_collect {
         #   extend the CVS summary of each file
         my @newinfo = ();
         foreach my $info (@cvsinfo) {
-            $info =~ m|^([^,]+),([^,]+),([^,]+)$|
-                || die "invalid loginfo argument `$info'";
+            $info =~ m|^([^,]+),([^,]+),([^,]+)|
+                || die "invalid loginfo argument `$info' while extending stock CVS information";
             my ($Is, $IV, $Iv) = ($1, $2, $3);
 
             my $It = '';
@@ -1508,7 +1514,7 @@ sub hook_loginfo_collect {
     my @newinfo = ();
     foreach my $info (@cvsinfo) {
         $info =~ m|^([^,]+),([^,]+),([^,]+),([^,]*),([^,]*)$|
-            || die "invalid loginfo argument `$info'";
+            || die "invalid loginfo argument `$info' while extending summary information";
         my ($Is, $IV, $Iv, $It, $Io) = ($1, $2, $3, $4, $5);
 
         #   fix branch/tag and accumulate information
@@ -1518,7 +1524,7 @@ sub hook_loginfo_collect {
         my $rcslog = '';
         if ($Io eq 'A' or $Io eq 'M') {
             if (not $RT->{useserver}) {
-                open(CVSS, "$RT->{cvs} -f -l -Q -n log -r$Iv $Is|");
+                open(CVSS, "$RT->{cvs} -f -l -Q -n log -r$Iv '$Is'|");
                 $rcslog .= $_ while (<CVSS>);
                 close(CVSS);
             }
@@ -1552,12 +1558,12 @@ sub hook_loginfo_collect {
             $Id = 'BLOB';
         }
         else {
-            if ($Io eq 'A' or $Io eq 'R') {
+            if ($Io eq 'A') {
                 open(FP, "<$Is"); 
                 my $l = 0;
                 $l++ while (<FP>);
                 close(FP);
-                $Id = sprintf("+%d/-%d", ($Io eq 'A' ? $l : 0), ($Io eq 'A' ? 0 : $l));
+                $Id = sprintf("+%d/-%d", $l, 0);
             }
             elsif ($Io eq 'M') {
                 if ($rcslog =~ m|\ndate:.*lines:\s*([\d \t+-]+)|s) {
@@ -1565,6 +1571,18 @@ sub hook_loginfo_collect {
                     $Id =~ s|\s+|/|g;
                 }
             }
+            elsif ($Io eq 'R') {
+                my $cvs = new CVS ($RT->{cvs}, $RT->{cvsroot});
+                $cvs->directory($cvsdir);
+                $cvs->entry($Is);
+                $cvs->arguments("-p", "-r$IV", $Is);
+                $cvs->send("update");
+                my $f = scalar $cvs->result;
+                $cvs->close;
+                my $l = 0;
+                $f =~ s|\n|$l++|sge;
+                $Id = sprintf("+%d/-%d", 0, $l);
+            }
         }
 
         #   determine change diff
@@ -1578,7 +1596,7 @@ sub hook_loginfo_collect {
                     "============================================================\n" .
                     "\$ cvs update -p -r$Iv $Is | uuencode $Is\n";
                 if (not $RT->{useserver}) {
-                    open(CVSS, "$RT->{cvs} -f -l -Q -n update -p -r$Iv $Is | uuencode $Is |");
+                    open(CVSS, "$RT->{cvs} -f -l -Q -n update -p -r$Iv '$Is' | uuencode '$Is' |");
                     $cvsdiff .= $_ while (<CVSS>);
                     close(CVSS);
                 }
@@ -1601,7 +1619,7 @@ sub hook_loginfo_collect {
                     "============================================================\n" .
                     "\$ cvs update -p -r$Iv $Is\n";
                 if (not $RT->{useserver}) {
-                    open(CVSS, "$RT->{cvs} -f -l -Q -n update -p -r$Iv $Is|");
+                    open(CVSS, "$RT->{cvs} -f -l -Q -n update -p -r$Iv '$Is'|");
                     $cvsdiff .= $_ while (<CVSS>);
                     close(CVSS);
                 }
@@ -1628,8 +1646,8 @@ sub hook_loginfo_collect {
                     "\$ cvs update -p -r$Iv $Is >$Is.new\n" .
                     "\$ diff -u $Is.old $Is.new\n";
                 if (not $RT->{useserver}) {
-                    system("$RT->{cvs} -f -l -Q -n update -p -r$IV $Is | uuencode $Is >$Is.old");
-                    system("$RT->{cvs} -f -l -Q -n update -p -r$Iv $Is | uuencode $Is >$Is.new");
+                    system("$RT->{cvs} -f -l -Q -n update -p -r$IV '$Is' | uuencode '$Is' >$Is.old");
+                    system("$RT->{cvs} -f -l -Q -n update -p -r$Iv '$Is' | uuencode '$Is' >$Is.new");
                 }
                 else {
                     my $cvs = new CVS ($RT->{cvs}, $RT->{cvsroot});
@@ -1658,7 +1676,7 @@ sub hook_loginfo_collect {
                 #   file was modified, so we show the changed contents only
                 my $d = '';
                 if (not $RT->{useserver}) {
-                    open(FP, "$RT->{cvs} -f -l -Q -n diff -u -r$IV -r$Iv $Is|");
+                    open(FP, "$RT->{cvs} -f -l -Q -n diff -u -r$IV -r$Iv '$Is'|");
                     $d .= $_ while (<FP>);
                     close(FP);
                 }
@@ -1671,9 +1689,10 @@ sub hook_loginfo_collect {
                     $d .= scalar $cvs->result;
                     $cvs->close;
                 }
+                my $Is_quoted = quotemeta($Is);
                 $d =~ s|^Index:.+?\ndiff\s+.*?\n||s;
-                $d =~ s|^(---\s+)$Is(\s+)|$1$cvsdir/$Is$2|m;
-                $d =~ s|^(\+\+\+\s+)$Is(\s+)|$1$cvsdir/$Is$2|m;
+                $d =~ s|^(---\s+)${Is_quoted}(\s+)|$1$cvsdir/$Is$2|m;
+                $d =~ s|^(\+\+\+\s+)${Is_quoted}(\s+)|$1$cvsdir/$Is$2|m;
                 $cvsdiff .=
                     "<Diff $cvsdir/$Is>\n" .
                      "Index: $cvsdir/$Is\n" .
@@ -1735,7 +1754,7 @@ sub hook_loginfo_accumulate {
     my $handle_max = undef;
     foreach my $cvsinfo (@cvsinfo) {
         $cvsinfo =~ m|^([^,]+),([^,]+),([^,]+),([^,]+),([^,]+),([^,]+),([^,]+),([^,]+)$|
-             || die "invalid loginfo argument `$cvsinfo'";
+             || die "invalid loginfo argument `$cvsinfo' while accumulating information";
         my ($Is, $IV, $Iv, $It, $Io, $Ik, $ID, $Id) = ($1, $2, $3, $4, $5, $6, $7, $8, $9);
         my $e = {};
         $e->{oldrev} = $IV;
@@ -1746,7 +1765,8 @@ sub hook_loginfo_accumulate {
         $e->{date}   = $ID;
         $e->{delta}  = $Id;
         $e->{diff} = '';
-        $cvsdiff =~ s|\n<Diff\s+$Is>\n(.+?\n)</Diff>|$e->{diff} = $1, ''|se;
+        my $Is_quoted = quotemeta($Is);
+        $cvsdiff =~ s|\n<Diff\s+${Is_quoted}>\n(.+?\n)</Diff>|$e->{diff} = $1, ''|se;
         $IN->{file}->{$Is} = $e;
         $handle_min = $ID if ($ID ne '' and (not defined($handle_min) or $handle_min > $ID));
         $handle_max = $ID if ($ID ne '' and (not defined($handle_max) or $handle_max < $ID));
@@ -1817,9 +1837,11 @@ sub hook_loginfo_process {
                 print "cvs commit: Mailing commit message to <$logurl>\n";
                 my $sm = new Sendmail ($RT, $logurl);
                 $sm->header('Subject', $subject);
-                if ($CF->{Logging}->{Report}->{$logtype}->{Details} eq 'diff:mime') {
-                    $sm->header('Content-Type', 
-                                "multipart/mixed; boundary=\"".$RT->{mimeboundary}."\"");
+                if (defined($CF->{Logging}->{Report}->{$logtype}->{Details})) {
+                    if ($CF->{Logging}->{Report}->{$logtype}->{Details} eq 'diff:mime') {
+                        $sm->header('Content-Type', 
+                                    "multipart/mixed; boundary=\"".$RT->{mimeboundary}."\"");
+                    }
                 }
                 $sm->body($logmsg);
                 $sm->close;
